/***************************************************************************
 *   Copyright (C) 2010 by Vladimir Jary                                   *
 *   <Vladimir.Jary@cern.ch>                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SPEEXPORTER_H
#define SPEEXPORTER_H


#include<QtCore/QString>
#include<QtCore/QDataStream>
#include<QtCore/QTextStream>

class QFile;

enum Platform { UNIX, DOS, MAC } ;

/// data format
enum Format { FLOAT, /// 32bit float
              INT32, /// 32bit signed integer
              INT16, /// 16bit signed integer
              UINT16 /// 16bit unsigned integer
          };

class SpeExporter
{
public:
    SpeExporter(QString input);
    ~SpeExporter();
    bool exportFile(QString ext);
    inline quint16 getX(){return x;}; /// Returns width in pixels
    inline quint16 getY(){return y;}; /// Returns height in pixels
    inline quint16 getNumberOfFrames(){return frames;}; ///  Returns number of frames stored in SPE file
    inline quint16 getDataFormat(){return format;}; /// Returns data format of opened SPE file
    void printHeader();
    void setNewLine(Platform platform);
private:
    template <typename TPixelFormat> void Allocate2DArray(TPixelFormat** &buff, quint32 rows, quint32 cols);
    template <typename TPixelFormat> void Free2DArray(TPixelFormat** &buff);
    template <typename TPixelFormat> void loadData(TPixelFormat** buff, QDataStream *in);
    template <typename TPixelFormat> void readPixel(QDataStream* instream, TPixelFormat &intensity);
    template <typename TPixelFormat> void export2txt(TPixelFormat intensity);
    template <typename TPixelFormat> void export2csv(TPixelFormat intensity);
    template <typename TPixelFormat> void export2asc(TPixelFormat intensity);
#ifdef USE_ROOT
    template <typename TPixelFormat> void export2root(TPixelFormat intensity);
#endif
    quint16 x, y; //dimension
    quint32 size; //size of frame
    quint16 frames; //number of frames
    quint16 format; //image format
    void deleteFile(QFile* file);
    void parseHeader();//get X and Y
    void constructOutputName(QString extension);
    void openStreams();
    void closeStreams();
    //output stream
    QFile *outFile;
    QTextStream *out;
    QString outputName;
    QString newLine;
    //input stream
    QFile *inFile;
    QDataStream *in;
    QString inputName;
    char separator;
};

#endif // SPEEXPORTER_H
